%Matched filtering demo
% Demonstration of matched filtering, ("Cross-correlation" or "Pulse
% Compression") which can be used to determine the location of multiple
% reflections of a particular waveform in a signal.
%
% LAST UPDATED by Andy French. January 2020.

function matched_filter_demo

%Define orginal signal(s)

%Times delays from transmission to reflection /microseconds
T = [3,5,7];

%Reflection amplitudes, relative to Tx
a = [0.5,0.7,0.2];

%Noise amplitude
noise = 1;

%Time interval between signal samples /microseconds
dt = 0.01;

%Chirp bandwidth /MHz
B = 1/dt;

%Chirp pulse length /microseconds
tau = 1.5;

%Gaussian signal width /microseconds
sigma = 0.1;


%

%Peak-finder dB above median power
dB_thresh = 15;

%dB scale for matched filter output
dBscale = [-60,5];

%Fontsize for graphs
fsize = 14;

%

%

%Time for transmitted signal
tt = 0 : dt : tau+dt;

%Time for received signal
t = 0 : dt : max(T)+tau;

%Generate Gaussian signal. Set to correspond to the middle of the chirp
%pulse. Set so energy of gaussian pulse is the same as the chirp. 
%Chirp pulse is set to have unit amplitude, so energy of chirp pulse is =
%tau. Area under gaussian curve is unity.
s_g_tx = gauss( tt ,tau/2,sigma,sqrt(tau) );
s_g_rx = zeros(size(t));
for n=1:length(T)
    s_g_rx = s_g_rx + gauss(t,T(n) + tau/2,sigma,a(n)*sqrt(tau) );
end
s_g_rx = s_g_rx + rand(1,length(t))*noise;

%Generate chirp signal
s_c_tx = chirp( B,tau,tt,0,0 ); s_c_rx = zeros( size(t) );
for n=1:length(T)
    s_c_rx = s_c_rx + a(n)*chirp( B,tau,t,T(n),0 );
end
s_c_rx = s_c_rx + ( rand(1,length(t)) - 0.5 ) * noise + 1i*( rand(1,length(t)) - 0.5 )*noise;

%Compute pulse compression for Gaussian and Chirp signals
[p_g,tpc] = xcor( s_g_tx, s_g_rx, dt );
[p_c,tpc] = xcor( s_c_tx, s_c_rx, dt );

%Normalize to peak
p_g = p_g/max(p_g);
p_c = p_c/max(p_c);

%Determine peak positions
[s_peak_g,tpeak_g,med_g,thresh_g,SNR_g] = peakfinder( p_g, tpc, dB_thresh );
[s_peak_c,tpeak_c,med_c,thresh_c,SNR_c] = peakfinder( p_c, tpc, dB_thresh );

%

%Gaussian signals: Plot signals, overlaid with pulse compression output
figure('color',[1 1 1],'name','gaussian')
subplot(2,1,1);
plot( tt, s_g_tx,'r',t, s_g_rx, 'b' ); legend('tx','rx','fontsize',fsize)
xlabel('time','fontsize',fsize); ylabel('|s|','fontsize',fsize)
title(['Gaussian Tx and Rx signals s(t): \sigma=',...
    num2str(sigma),', a=',vec2str(a),', \Deltat=',vec2str(T),...
    ', Noise=',num2str(noise)],'fontsize',fsize);
grid on; box on; set(gca,'fontsize',fsize); xlim([0,max(t)])
subplot(2,1,2)
plot( tpc, dB( p_g.*conj(p_g) ), 'k' ); hold on;
plot( [0,max(t)], dB(med_g^2)*[1,1],'m' );
plot( [0,max(t)], dB(thresh_g^2)*[1,1],'b' );
plot( tpeak_g, dB(s_peak_g.^2),'r*' );
xlabel('time','fontsize',fsize); ylabel('10log_{10}|s|^2 /dB','fontsize',fsize)
title(['Matched filter output: Peak SNR=',vec2str(SNR_g),' dB'],'fontsize',fsize);
grid on; box on; set(gca,'fontsize',fsize); xlim([0,max(t)]); ylim(dBscale);
print( gcf, 'gauss.png','-r300','-dpng' ); close(gcf);

%Chirp signals: Plot signals, overlaid with pulse compression output
figure('color',[1 1 1],'name','chirp')
subplot(2,1,1);
plot( tt, abs(s_c_tx),'r',t, abs(s_c_rx), 'b' ); legend('tx','rx','fontsize',fsize)
xlabel('time','fontsize',fsize); ylabel('|s|','fontsize',fsize)
title(['Chirp Tx and Rx signals s(t): \tau=',...
    num2str(tau),', B=',num2str(B),', a=',vec2str(a),', \Deltat=',vec2str(T),...
    ', Noise=',num2str(noise)],'fontsize',fsize);
grid on; box on; set(gca,'fontsize',fsize); xlim([0,max(t)]);
subplot(2,1,2)
plot( tpc, dB( p_c.*conj(p_c) ),'k' ); hold on;
plot( [0,max(t)], dB(med_c^2)*[1,1],'m' );
plot( [0,max(t)], dB(thresh_c^2)*[1,1],'b' );
plot( tpeak_c, dB(s_peak_c.^2),'r*' );
xlabel('time','fontsize',fsize)
ylabel('10log_{10}|s|^2 /dB','fontsize',fsize)
title(['Matched filter output: Peak SNR=',vec2str(SNR_c),' dB'],'fontsize',fsize);
grid on; box on; set(gca,'fontsize',fsize); xlim([0,max(t)]); ylim(dBscale);
print( gcf, 'chirp.png','-r300','-dpng' );  close(gcf);

%Chirp signals: Plot real and imaginary parts of signal
figure('color',[1 1 1],'name','chirp')
subplot(2,1,1);
plot( tt, real(s_c_tx),'r',t, real(s_c_rx), 'b' ); legend('tx','rx','fontsize',fsize)
xlabel('time','fontsize',fsize); ylabel('Re(s)','fontsize',fsize);
ylimit = max(abs(s_c_tx)); ylim([-ylimit,ylimit]);
title(['Chirp Tx and Rx signals s(t): \tau=',...
    num2str(tau),', B=',num2str(B),', a=',vec2str(a),', \Deltat=',vec2str(T),...
    ', Noise=',num2str(noise)],'fontsize',fsize);
grid on; box on; set(gca,'fontsize',fsize); xlim([0,max(t)])
subplot(2,1,2);
plot( tt, imag(s_c_tx),'r',t, imag(s_c_rx), 'b' ); legend('tx','rx','fontsize',fsize)
xlabel('time','fontsize',fsize); ylabel('Im(s)','fontsize',fsize);
ylim([-ylimit,ylimit]);
grid on; box on; set(gca,'fontsize',fsize); xlim([0,max(t)])
print( gcf, 'chirp re and im.png','-r300','-dpng' );  close(gcf);

%%

%Gaussian signal function (not area under curve is unity)
function y = gauss(t,t0,sigma,a)
y = ( a/sqrt( 2*pi*(sigma^2) ) ) * exp( - ( (t - t0).^2 )/( 2*sigma^2 ) );

%%

%chirp
% Function that computes samples of a linear chirp pulse.
% One assumes that this has been mixed down to the carrier frequency fc.
% B     - Chirp bandwidth /MHz
% tau   - Chirp length /microseconds
% t     - Time /microseconds (can be an vectau)
% t0    - Time of reception of leading edge of chirp /microseconds
% fc    - Carrier frequency of chirp /MHz
% s     - Samples of chirp at times t
function s=chirp(B,tau,t,t0,fc)
alpha = pi*B/tau; beta = -pi*B; gamma = -pi*tau*(fc-B/4);
PHI = alpha*(t-t0).^2 + beta*(t-t0) + gamma;
s=exp(1i*PHI);
s( ( t<t0 ) | ( t > (t0+tau) ) ) = 0;

%%

%xcor
% Function which computes cross-correlation with transmitted signal tx
% and received signal rx. 
function [xc,t] = xcor( tx,rx,dt )

%Time reverse and complex conjugate transmitted signal and pad with zeros.
%Also apply a window to reduce effects of the finite width
%DFT as compared to a full Fourier Transform.
N = round( length(rx)/2 );
w = hamming( length(tx) );
x = [ zeros(1,N),fliplr( conj( tx.*w ) ),zeros(1,N) ];

%Pad x or rx with zeros
if length(x)>length(rx)
    y = [rx,zeros( 1,length(x)-length(rx) )];
elseif length(rx)>length(x)
    x = [x,zeros( 1,length(rx)-length(x) )];
end

%Perform convolution of x and y using FFT functions
xc=ifftshift( ifft( fft(x).*fft(y) ) );

%Generate time vectau for matched filter output
t = 0:dt:( length(xc)-1 )*dt;

%%

%peakfinder
% Function which yields peak times and amplitudes for a signal x(t)
% that are above a fixed number of dB above the median signal power
function [xpeak,tpeak,med,thresh,SNR] = peakfinder( x,t, dB_thresh )

%Compute signal power
p = x.*conj(x);

%Zero x values that correspond to powers below threshold
x( 10*log10(p) < ( dB_thresh + 10*log10( median(p) ) ) ) = 0;

%Step through x and determine peaks
xpeak = []; tpeak = [];
for n=2:length(x)-1
    if ( x(n) > x(n-1) ) && ( x(n) > x(n+1) )
        xpeak = [ xpeak, x(n) ];
        tpeak = [ tpeak, t(n) ];
    end
end

%Determine amplitude thresholds
med = sqrt( median(p) );
thresh = sqrt( median(p)*10^(dB_thresh/10) );

%Determine SNR values of peaks /dB
SNR = 20*log10( abs(xpeak)/med );

%%

%hamming
% Hamming windowing function for signal of N elements. The main role
% of the window (as with all windows) is to damp out the effects of the
% Gibbs phenomenon that results from truncation of an infinite series (e.g.
% a Discrete Fourier Transform). Note this window is complex
% and can therefore only really be visualised when the amplitude of the 
% window DFT is plotted (in dB).
function w = hamming(N)
w = 0.54 - 0.46*cos( 2*pi*( 0:(N-1) )/N );

%%

%dolphcheb
% Function that computes Dolph-Chebyshev windowing function. 
% N - Length of window sl_level. This difference between peak and
%     sidelobes /dB in the DFT of the window.
% w - Window 'weights'. These are real, positive numbers.
function w=dolphcheb(N,sl_level)
gamma=sl_level/10;
if N>1
    M=N-1;
    alpha=cosh(acosh(10^gamma)/M);
    for n=1:M
        A(n)=((-1)^(n-1))*cos(M*acos(alpha*cos(pi*(n-1)/M)));
    end
    B=fft(A);
    B(1)=0.5*B(1);
    B(N)=B(1); 
    w=B/max(B);
else
    w=1;    
end

%%

%vec2str
% Does a slightly nicer job of displaying a vectau of values e.g. [1,2,3]
% as a string
function s = vec2str(v)
s = '[';
for n=1:length(v)
    s = [s,num2str(v(n),3),','];
end
s(end) = ']';

%%

%dB
% Convert to decibels. Typically used for signal power.
function y = dB(x)
y = 10*log10(x);

%End of code