%phased_array_linear_1D
% Model of a 1D phased linear array antenna. N approximately isotropic
% radiating elements are arranged in a nominally equi-spaced line. The
% far-field beam pattern is determined in dB relative to 'boresight' which
% is the perpendicular to the centre of the array. An amplitude and
% phase weighting is applied to the array elements in order to steer the
% beam by angle phi from boresight.
%
% LAST UPDATED by Andy French Nov 2019

function phased_array_linear_1D
global d

%Number of array elements
d.N = 32; d.dN = 1;

%Spacing (in wavelengths) of antenna elements
d.s = 0.6; d.ds = 0.05;

%Random error (in wavelengths) of x,y coordinates of array elements
d.dxy = 0.0; d.ddxy = 0.001;

%Initial beam angle /deg and angular deviation following left-right arrow key
%presses
d.phi = 30; d.dphi = 1;

%Target range in wavelengths
d.R = 100; d.dR = 1;

%

%Beam pattern dB scale
dBscale = [-60,50];

%Beam pattern angle scales /deg
angle_scale = [-180,180];

%Frequency /Hz
f = 3e9;

%Speed of radiation c /ms^-1
c = 2.998e8;

%Number of beam angle computation points
d.K = 1000;

%Fontsize for plots
d.fsize = 18;

%

%Compute initial model run
[x,y,X,Y,theta,psi,beam_dB,sidelobe_dB,xp,yp] =...
    phased_array_linear_1D_model( d.phi, d.N, d.s, d.dxy, d.K, d.R );

%Plot beam pattern
d.fig = figure('name','Phased array model','color',[1 1 1],'KeyPressFcn',@keypressfunc );
d.beam_dB_plot = plot( theta, beam_dB,'b' ); hold on;
d.beam_dB_max_plot = plot( theta, ones(1,d.K)*max(beam_dB),'r-' );
d.beam_sidelobe_plot = plot( theta, ones(1,d.K)*sidelobe_dB,'g' );
d.peaks_plot = plot(xp,yp,'r*');
d.peak_angle_plot = plot( [d.phi,d.phi], dBscale, 'r' );
xlim(angle_scale); ylim(dBscale); axis manual;
xlabel('Beam angle /deg', 'fontsize', d.fsize );
ylabel('Beam pattern power /dB', 'fontsize', d.fsize );
set(gca, 'fontsize', d.fsize ); grid on;
tit = ['Linear array N=',num2str(d.N),', phi=',num2str(d.phi),', s=',...
    num2str(d.s),', dxy=',num2str(d.dxy),', R=',num2str(d.R)...
    ', P:SL=',num2str(max(beam_dB),3),':',num2str(sidelobe_dB,3)];
d.title = title(tit);

%%

%phased_array_linear_1D_model
function [x,y,X,Y,theta,psi,beam_dB,sidelobe_dB,xp,yp] =...
    phased_array_linear_1D_model( phi, N, s, dxy, K, R )

%Determine array element x,y positions
x = -N*s/2 + (0:N-1)*s;
y = zeros(1,N);

%Add Gaussian positional errors
x = x + dxy * ( sqrt(2)*erfinv( 2*rand(1,N) - 1 ) );
y = y + dxy * ( sqrt(2)*erfinv( 2*rand(1,N) - 1 ) );

%Define beam angles /deg, clockwise from boresight
theta = linspace(-180,180,K);

%Define wavefield target positions in wavelengths
X = R*sin( theta*pi/180 ); Y = R*cos( theta*pi/180 );

%Define amplitude and phase weightings of the array elements

%Hamming amplitude weighting
a = 0.54 - 0.46*cos( 2*pi*( 0:(N-1) )/N );

%Phase weighting
alpha = pi/2 - phi*pi/180 -atan2( y, x) ;
phase = 2*pi*sqrt( x.^2 + y.^2 ).*cos( alpha );

%Costruct complex array element weighting
w = a.*exp( 1i*phase );

%Compute electric field strength psi at (R,theta)
psi = zeros(1,K);
for k=1:K
    for n=1:N
        dR = sqrt( ( X(k)-x(n) )^2 +  ( Y(k)-y(n) )^2 );
        psi(k) = psi(k) + exp( 2*pi*1i*dR ) * w(n);
    end
end

%Calculate beam pattern power in dB
beam_dB = 10*log10( psi.*conj(psi) );

%Determine peaks
[xp,yp] = peakfindergeneral(theta,beam_dB);

%Determine sidelobe level i.e. the second highest peak. Ignore grating
%lobes, hence the 0.9 factor.
sidelobe_dB = max( yp( yp < 0.9*max(yp) ) );
if isempty(sidelobe_dB); sidelobe_dB = NaN; end

%%

%peakfindergeneral
% Finds coordinates of peaks of curve described by y(x)
function [xp,yp] = peakfindergeneral(x,y)
%Sort x in ascending order
[x,i] = sort(x); y = y(i);

%Step through y and determine local peaks
xp =[]; yp =[];
for n=2:length(x)-1
    if ( y(n) > y(n-1) ) && ( y(n) > y(n+1) )
        xp = [xp,x(n)]; yp = [yp,y(n)];
    end
end

%Sort by peak size
[yp,i] = sort( yp, 2, 'descend' );
xp = xp(i);

%%

%Key press handler
function keypressfunc(hObject, eventdata, handles)
global d

%Get the string of the current key press
s = get( d.fig,'currentkey');

%Actions, depending on the key pressed...
if ( strcmp(s,'q') == 1 ) || ( strcmp(s,'Q') == 1 )
    %Quit
    close(d.fig); return;
elseif ( strcmp(s,'p') == 1 )
    %Print figure
    filename =['Linear array N=',num2str(d.N),', phi=',num2str(d.phi),', s=',...
        num2str(d.s),', dxy=',num2str(d.dxy),', R=',num2str(d.R)];
    filename = [ strrep(filename,'.','p'),'.png'];
    print( d.fig, filename,'-dpng','-r300' );
elseif ( strcmp(s,'leftarrow') == 1 )
    %Decrement scan angle phi
    d.phi = d.phi - d.dphi;
elseif ( strcmp(s,'rightarrow') == 1 )
    %Increment scan angle phi
    d.phi = d.phi + d.dphi;
elseif ( strcmp(s,'downarrow') == 1 )
    %Decrement number of array elements N
    d.N = d.N - d.dN;
    if d.N < 1; d.N = 1; end
elseif ( strcmp(s,'uparrow') == 1 )
    %Increment number of array elements N
    d.N = d.N + d.dN;
elseif ( strcmp(s,'z') == 1 )
    %Decrement target range R in wavelengths
    d.R = d.R - d.dR;
    if d.R < 1; d.R = 1; end
elseif ( strcmp(s,'a') == 1 )
    %Increment target range R in wavelengths
    d.R = d.R + d.dR;
elseif ( strcmp(s,'x') == 1 )
    %Decrement array separtion s in wavelengths
    d.s = d.s - d.ds;
    if d.s<0; d.s = 0; end
elseif ( strcmp(s,'s') == 1 )
    %Increment array separtion s in wavelengths
    d.s = d.s + d.ds;
elseif ( strcmp(s,'c') == 1 )
    %Decrement array position error dxy in wavelengths
    d.dxy = d.dxy - d.ddxy;
    if d.dxy<0; d.dxy = 0; end
elseif ( strcmp(s,'d') == 1 )
    %Increment array position error dxy in wavelengths
    d.dxy = d.dxy + d.ddxy;
elseif ( strcmp(s,'l') == 1 )
    %Plot linear array element positions
    figure('color',[1 1 1])
    [x,y,X,Y,theta,psi,beam_dB,sidelobe_dB,xp,yp] =...
    phased_array_linear_1D_model( d.phi, d.N, d.s, d.dxy, d.K, d.R );
    plot( x,y,'b',x,y,'r*'); axis tight;
    tit = ['Linear array N=',num2str(d.N),', phi=',num2str(d.phi),', s=',...
    num2str(d.s),', dxy=',num2str(d.dxy),', R=',num2str(d.R)];
    title(tit,'fontsize',d.fsize); 
    grid on; set(gca,'fontsize',d.fsize); 
    xlabel('x /\lambda','fontsize',d.fsize);
    ylabel('y /\lambda','fontsize',d.fsize);
    filename =['Linear array xy N=',num2str(d.N),', phi=',num2str(d.phi),', s=',...
        num2str(d.s),', dxy=',num2str(d.dxy),', R=',num2str(d.R)];
    filename = [ strrep(filename,'.','p'),'.png'];
    print( gcf, filename,'-dpng','-r300' );
    close(gcf); figure(d.fig);
end

%Update model
[x,y,X,Y,theta,psi,beam_dB,sidelobe_dB,xp,yp] =...
    phased_array_linear_1D_model( d.phi, d.N, d.s, d.dxy, d.K, d.R );

%Update plot
set( d.beam_dB_plot,'ydata',beam_dB);
set( d.beam_dB_max_plot,'ydata', ones(1,d.K)*max(beam_dB) );
set( d.beam_sidelobe_plot, 'ydata', ones(1,d.K)*sidelobe_dB );
set( d.peaks_plot,'xdata',xp,'ydata',yp );
set( d.peak_angle_plot, 'xdata',[d.phi,d.phi] );
tit = ['Linear array N=',num2str(d.N),', phi=',num2str(d.phi),', s=',...
    num2str(d.s),', dxy=',num2str(d.dxy),', R=',num2str(d.R),...
    ', P:SL=',num2str(max(beam_dB),3),':',num2str(sidelobe_dB,3)];
set( d.title, 'string',tit );

%End of code