%phased_array_rectangular
% Model of a 2D rectangular phased array antenna. N*N approximately isotropic
% radiating elements are arranged in a nominally equi-spaced grid. The
% far-field beam pattern is determined in dB relative to 'boresight' which
% is the perpendicular to the centre of the array. An amplitude and
% phase weighting is applied to the array elements in order to steer the
% beam by azimuth phi and elevation elev from boresight.
%
% LAST UPDATED by Andy French Nov 2019

function phased_array_rectangular
global d

%Number of array elements (note the grid is N^2)
d.N = 16; d.dN = 1;

%Spacing (in wavelengths) of antenna elements (assume same in x and y)
d.s = 0.5; d.ds = 0.05;

%Random error (in wavelengths) of x,y coordinates of array elements
d.dxy = 0.0; d.ddxy = 0.001;

%Initial beam angle /deg and angular deviation following left-right arrow key
%presses
d.phi = 0; d.dphi = 1;
d.elev = 0; d.delev = 1;

%Target range in wavelengths
d.R = 100; d.dR = 1;

%

%Beam pattern dB scale
dBscale = [-60,50];

%Beam pattern angle scales /deg
azi_angle_scale = [-180,180];
elev_angle_scale = [-90,90];

%Frequency /Hz
f = 3e9;

%Speed of radiation c /ms^-1
c = 2.998e8;

%Number of beam angle computation points
d.K = 200;

%Fontsize for plots
d.fsize = 18;

%

%Compute initial model run
[x,y,z,X,Y,Z,azi,elev,psi,beam_dB] =...
    phased_array_rectangular_model( d.phi, d.elev, d.N, d.s, d.dxy, d.K, d.R );

%Plot beam pattern
d.fig = figure('name','Phased array model','color',[1 1 1],'KeyPressFcn',@keypressfunc );
d.beam_dB_plot = surf( azi, elev, beam_dB ); view(2);
shading interp; interp_colormap(500);
colorbar; caxis(dBscale);
xlim(azi_angle_scale); ylim(elev_angle_scale); axis equal; axis manual;
xlabel('Azimuth /deg', 'fontsize', d.fsize );
ylabel('Elevation /deg', 'fontsize', d.fsize );
set(gca, 'fontsize', d.fsize ); grid on;
tit = ['Rectangular array N=',num2str(d.N),', phi=',num2str(d.phi),...
    ', elev=',num2str(d.elev),', s=',...
    num2str(d.s),', dxy=',num2str(d.dxy),', R=',num2str(d.R)];
d.title = title(tit);

%%

%phased_array_rectangular_model
function [x,y,z,X,Y,Z,azi,elev,psi,beam_dB] =...
    phased_array_rectangular_model( a, e, N, s, dxy, K, R )

%Convert scan angles to radians
a = a*pi/180; e = e*pi/180;

%Determine array element x,y positions
x = linspace(-(N-1)*s/2,(N-1)*s/2,N);
y = linspace(-(N-1)*s/2,(N-1)*s/2,N);
[x,y] = meshgrid(x,y);
x = reshape( x, [1,N^2] ); y = reshape( y, [1,N^2] );
z = zeros(1,N^2);

%Add Gaussian positional errors
x = x + dxy * ( sqrt(2)*erfinv( 2*rand(1,N^2) - 1 ) );
y = y + dxy * ( sqrt(2)*erfinv( 2*rand(1,N^2) - 1 ) );

%Define beam angles /deg, clockwise from boresight
azi = linspace(-180,180,K);
elev = linspace(-90,90,K);
[azi,elev] = meshgrid( azi,elev );

%Define wavefield target positions in wavelengths
X = R*cos( elev*pi/180 ).*sin( azi*pi/180 );
Z = R*cos( elev*pi/180 ).*cos( azi*pi/180 );
Y = R*sin( elev*pi/180 );

%Define amplitude and phase weightings of the array elements

%Hamming amplitude weighting
Ax = 0.54 - 0.46*cos( 2*pi*( 0:(N-1) )/N );
Ay = 0.54 - 0.46*cos( 2*pi*( 0:(N-1) )/N );
[Ax,Ay] = meshgrid(Ax,Ay); A = Ax.*Ay; A = reshape(A,[1,N^2]);

%Phase weighting
phase = 2*pi*( sin(a)*x + sin(e)*y );

%Costruct complex array element weighting
w = A.*exp( 1i*phase );

%Compute electric field strength psi at (R,theta)
psi = zeros(K,K);
for n=1:N^2
    dR = sqrt( ( X-x(n) ).^2 +  ( Y-y(n) ).^2 + ( Z-z(n) ).^2 );
    psi = psi + exp( 2*pi*1i*dR ) * w(n);
end

%Calculate beam pattern power in dB
beam_dB = 10*log10( psi.*conj(psi) );

%%

%Key press handler
function keypressfunc(hObject, eventdata, handles)
global d

%Get the string of the current key press
s = get( d.fig,'currentkey');

%Actions, depending on the key pressed...
if ( strcmp(s,'q') == 1 ) || ( strcmp(s,'Q') == 1 )
    %Quit
    close(d.fig); return;
elseif ( strcmp(s,'p') == 1 )
    %Print figure
    filename =['Rectangular array N=',num2str(d.N),', phi=',num2str(d.phi),', s=',...
        num2str(d.s),', dxy=',num2str(d.dxy),', R=',num2str(d.R)];
    filename = [ strrep(filename,'.','p'),'.png'];
    print( d.fig, filename,'-dpng','-r300' );
elseif ( strcmp(s,'leftarrow') == 1 )
    %Decrement scan angle phi
    d.phi = d.phi - d.dphi;
elseif ( strcmp(s,'rightarrow') == 1 )
    %Increment scan angle phi
    d.phi = d.phi + d.dphi;
elseif ( strcmp(s,'downarrow') == 1 )
    %Decrement scan angle elev
    d.elev = d.elev - d.delev;
elseif ( strcmp(s,'uparrow') == 1 )
    %Increment scan angle elev
    d.elev = d.elev + d.delev;
elseif ( strcmp(s,'space') == 1 )
    %Decrement number of array elements N
    d.N = d.N - d.dN;
    if d.N < 1; d.N = 1; end
elseif ( strcmp(s,'n') == 1 )
    %Increment number of array elements N
    d.N = d.N + d.dN;
elseif ( strcmp(s,'z') == 1 )
    %Decrement target range R in wavelengths
    d.R = d.R - d.dR;
    if d.R < 1; d.R = 1; end
elseif ( strcmp(s,'a') == 1 )
    %Increment target range R in wavelengths
    d.R = d.R + d.dR;
elseif ( strcmp(s,'x') == 1 )
    %Decrement array separtion s in wavelengths
    d.s = d.s - d.ds;
    if d.s<0; d.s = 0; end
elseif ( strcmp(s,'s') == 1 )
    %Increment array separtion s in wavelengths
    d.s = d.s + d.ds;
elseif ( strcmp(s,'c') == 1 )
    %Decrement array position error dxy in wavelengths
    d.dxy = d.dxy - d.ddxy;
    if d.dxy<0; d.dxy = 0; end
elseif ( strcmp(s,'d') == 1 )
    %Increment array position error dxy in wavelengths
    d.dxy = d.dxy + d.ddxy;
elseif ( strcmp(s,'l') == 1 )
    %Plot linear array element positions
    figure('color',[1 1 1])
    [x,y,z,X,Y,Z,azi,elev,psi,beam_dB] =...
        phased_array_rectangular_model( d.phi, d.elev, d.N, d.s, d.dxy, d.K, d.R );
    plot( x,y,'r*'); axis tight;
    tit = ['Rectangular array N=',num2str(d.N),', phi=',num2str(d.phi),', s=',...
        num2str(d.s),', dxy=',num2str(d.dxy),', R=',num2str(d.R)];
    title(tit,'fontsize',d.fsize);
    grid on; set(gca,'fontsize',d.fsize);
    xlabel('x /\lambda','fontsize',d.fsize);
    ylabel('y /\lambda','fontsize',d.fsize);
    filename = ['Rectangular array xy N=',num2str(d.N),', phi=',num2str(d.phi),...
        ', elev=',num2str(d.elev),', s=',num2str(d.s),...
        ', dxy=',num2str(d.dxy),', R=',num2str(d.R)];
    filename = [ strrep(filename,'.','p'),'.png'];
    print( gcf, filename,'-dpng','-r300' );
    close(gcf); figure(d.fig);
end

%Update model
[x,y,z,X,Y,Z,azi,elev,psi,beam_dB] =...
    phased_array_rectangular_model( d.phi, d.elev, d.N, d.s, d.dxy, d.K, d.R );

%Update plot
set( d.beam_dB_plot,'zdata',beam_dB);
tit = ['Rectangular array N=',num2str(d.N),', phi=',num2str(d.phi),...
    ', elev=',num2str(d.elev),', s=',num2str(d.s),...
    ', dxy=',num2str(d.dxy),', R=',num2str(d.R)];
set( d.title, 'string',tit );

%%

%interp_colormap
% Function which interpolates current colourmap to yield better graduated
% shading. N is number of possible colours.
function interp_colormap(N)

%Get current colourmap
map = colormap;

%Initialise new colormap
new_map = ones(N,3);

%Get size of current colormap and initalise red,green,blue vectors
dim = size(map);
R = ones(1,dim(1));
G = ones(1,dim(1));
B = ones(1,dim(1));
RR = ones(1,N);
GG = ones(1,N);
BB = ones(1,N);

%Populate these with current colormap
R(:) = map(:,1);
G(:) = map(:,2);
B(:) = map(:,3);

%Interpolate to yield new colour map
x = linspace( 1, dim(1), N );
RR = interp1( 1:dim(1), R, x );
GG = interp1( 1:dim(1), G, x );
BB = interp1( 1:dim(1), B, x );
new_map(:,1) = RR(:);
new_map(:,2) = GG(:);
new_map(:,3) = BB(:);

%Set colormap to be new map
colormap( new_map );

%End of code