%radar_equation
% Plots signal to noise ratio (SNR) vs range for a generic primary radar
% system i.e. an evaluation of the Radar Equation.
%
% LAST UPDATED by Andy French January 2020
%
% Similar parameters to my PhD thesis pp218.
% Note: dB(x) = 10*log10(x)

function radar_equation

%Antenna cross sectional area /m^2
A = 4;

%Antenna efficiency eta  (Gain at boresight = 4*pi * eta / beawidth^2 )
eta = 0.5;

%Pulse integration efficiency Ei(n)
Ei = 1;

%Number of pulses n integrated
n = 8;

%Pulse width /s
tau = 25.6e-6;

%Total transmitted power /W
Pt = [10,50,100]*1e3;

%Target Radar Cross Section (RCS) / m^2
RCS = 20;

%Carrier frequency /Hz
fc = 3e9;

%Receiver noise figure /K
T0Nf = 500;

%Total loss factor (transmit, recieve, signal processing) /dB
L = 10;

%Propagation loss factor abs( F^2 ) /dB (multipath would perhaps be the most
%significant here)
L_prop = 0;

%Range range /km
Rmin=10; Rmax = 200; N = 1000;

%Fontsize for graphs
fsize = 18;

%

%Evaluate Radar equation
[R,SNR_dB_Pt1] = radar_eq( Rmin, Rmax, A, n, Ei, tau, Pt(1), RCS, fc, L, L_prop, T0Nf,N );
[R,SNR_dB_Pt2] = radar_eq( Rmin, Rmax, A, n, Ei, tau, Pt(2), RCS, fc, L, L_prop, T0Nf,N );
[R,SNR_dB_Pt3] = radar_eq( Rmin, Rmax, A, n, Ei, tau, Pt(3), RCS, fc, L, L_prop, T0Nf,N );

%Plot 10*log10( SNR ) vs range. Note S (signal) and N (noise) are powers.
plot( R, SNR_dB_Pt1,R, SNR_dB_Pt2,R, SNR_dB_Pt3,'linewidth',2 ); hold on;
xlabel( 'Range R /km','fontsize',fsize );
for i=1:length(Pt); lgnd_str{i} = [num2str(Pt(i)/1000),'kW']; end;
ylabel(' 10log_{10}( Signal to Noise ratio ) /dB','fontsize',fsize )
legend( lgnd_str,'fontsize',fsize );
title( { ['Radar Equation: P_t=',vec2str(Pt/1000),'kW, \sigma=',...
    num2str(RCS),'m^2, f_c=',num2str(fc/1e9),'GHz, A=',num2str(A),'m^2'],...
    ['\eta=',num2str(eta),', E_i=',num2str(Ei),', n=',num2str(n),...
    ', \tau=',num2str(tau/1e-6),'\mus, T_0N_f=',num2str(T0Nf),...
    'K, L=',num2str(L),'dB.'] },'fontsize',fsize );
set(gca,'fontsize',fsize); grid on;
print( gcf,'radar equation.png','-r300','-dpng' );
close(gcf);

%%

%Radar equation
function [R,SNR_dB] = radar_eq( Rmin, Rmax, A, n, Ei, tau, Pt, RCS, fc, L, L_prop, T0Nf,N )

%Bolzmann's constant /JK^-1
kB = 1.38e-23;

%Speed of light /ms^-1
c = 2.998e8;

%Range /km
R = linspace(Rmin,Rmax,N);

%SNR = k / R^4 in dB
k = A^2 * n * Ei * tau * Pt * RCS * fc^2 * 10^(L_prop/10) /...
    ( 4*pi * c^2 * 10^(L/10) * kB * T0Nf );
SNR_dB = 10*log10( k*( (R*1e3).^-4 ) );

%%

%vec2str
% Does a slightly nicer job of displaying a vectau of values e.g. [1,2,3]
% as a string
function s = vec2str(v)
s = '[';
for n=1:length(v)
    s = [s,num2str(v(n),3),','];
end
s(end) = ']';

%End of code